#' Plots vaf distribution of genes
#' @description Plots vaf distribution of genes as a boxplot or violinplot.
#'
#' @param maf an \code{\link{MAF}} object generated by \code{\link{read.maf}}
#' @param vafCol manually specify column name for vafs. Default looks for column 't_vaf'
#' @param genes specify genes for which plots has to be generated
#' @param density logical whether to plot density plot of vaf
#' @param violin if TRUE plots violin plot
#' @param top if \code{genes} is NULL plots top n number of genes. Defaults to 5.
#' @return ggplot object which can be further modified.
#' @examples
#' laml.maf <- system.file("extdata", "tcga_laml.maf.gz", package = "maftools")
#' laml <- read.maf(maf = laml.maf, removeSilent = TRUE, useAll = FALSE)
#' plotVaf(maf = laml, vafCol = 'i_TumorVAF_WU')
#'
#' @export

plotVaf = function(maf, vafCol = NULL, genes = NULL, density = FALSE, violin = FALSE, top = 5){

  dat = maf@data

  if(!'t_vaf' %in% colnames(dat)){
    if(is.null(vafCol)){
      print(colnames(dat))
      stop('t_vaf field is missing. Use vafCol to manually specify vaf column name.')
    }else{
      colnames(dat)[which(colnames(dat) == vafCol)] = 't_vaf'
    }
  }

  if(is.null(genes)){
    genes = maf@gene.summary[1:top, Hugo_Symbol]
  }

  #dat.genes = data.frame(dat[dat$Hugo_Symbol %in% genes])
  #suppressMessages(datm <- melt(dat.genes[,c('Hugo_Symbol', 't_vaf')]))
  dat.genes = dat[dat$Hugo_Symbol %in% genes]
  suppressWarnings(datm <- data.table::melt(data = dat.genes[,.(Hugo_Symbol, t_vaf)]))
  #remove NA from vcf
  #datm = datm[!is.na(datm$value),]
  #maximum vaf

  if(max(datm$value, na.rm = TRUE) > 1){
    datm$value = datm$value/100
  }

  if(density){
    gg = ggplot(datm, aes(value, color = Hugo_Symbol))+geom_density(size = 1)+ cowplot::theme_cowplot() +
      geom_point(aes(y = 0, x = value), size = 3, alpha = 0.6)+xlim(0, 1)+theme(legend.position = 'bottom')+cowplot::background_grid(major = 'xy')
  } else{
    if(violin){
      gg = ggplot(data = datm, aes(x = Hugo_Symbol, y = value, color = Hugo_Symbol))+geom_violin()+ylim(0, 1)+ cowplot::theme_cowplot()+
        theme(legend.position = 'none')+ylab('vaf')+ cowplot::background_grid(major = 'xy')
    }else{
      gg = ggplot(data = datm, aes(x = Hugo_Symbol, y = value, color = Hugo_Symbol))+geom_boxplot()+ylim(0, 1)+cowplot::theme_cowplot()+
        theme(legend.position = 'none')+ylab('vaf')+cowplot::background_grid(major = 'xy')
    }
  }

  print(gg)
  return(gg)
}
